﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using DomainModel;
using System.Data;

namespace Tests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class TestOptimisticCoarseGrainLocking
    {
        public TestOptimisticCoarseGrainLocking()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
         //Use TestInitialize to run code before running each test 
         [TestInitialize()]
         public void MyTestInitialize()
         {
             Util.TestInitialise();
         }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestCreateOrder()
        {
            //
            // TODO: Add test logic	here
            //

            Util.CreateEmptyOrder();


            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                Assert.AreEqual(0, order.Version.Value, "Expected version 0 ");

                ctx.Submit();
            }

        }

        [TestMethod]
        public void TestMusLoadVersion()
        { //
            // TODO: Add test logic	here
            //

            Util.CreateEmptyOrder();


            try
            {
                using (AcmeWidgets ctx = new AcmeWidgets())
                {
                    Order order = ctx.OrderSet.First();
                    order.Shipped = true;

                    ctx.Submit();
                }
                Assert.Fail("Should have deteted that the version entity has not been loaded");
            }
            catch (SharedVersionException expected)
            {

            }

           

        }


        [TestMethod]
        public void TestNoChangesDetected()
        { //
            // TODO: Add test logic	here
            //

            Util.CreateEmptyOrder();


            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                AssertNoChangesYet(ctx);

                order.OrderItems.Add(new OrderItem()
                {
                    Products = ctx.ProductSet.First(),
                    Quantity = 1,
                });


                ctx.Submit();
            }

             using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet
                    .Include("Version")
                    .Include("OrderItems")
                    .Include("OrderItems.Version")
                    .First();

                    AssertNoChangesYet(ctx);

             }


        }

        [TestMethod]
        public void TestDeleteOrder()
        {
             Util.CreateEmptyOrder();

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();
                var products = ctx.ProductSet.ToList();

                order.OrderItems.Add(new OrderItem()
            {
                Quantity = 1,
                Products = products[0],
            });

                ctx.Submit();
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                AssertNoChangesYet(ctx);


                // Note using a strongly typed entity delete method
                // so that it tidies up the version table
                // could consider using a db trigger here
                ctx.DeleteOrder(order);

                ctx.Submit();
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Assert.AreEqual(0, ctx.OrderSet.Include("Version").Count());

                Assert.AreEqual(0, ctx.SharedVersionSet.Count());
            }
        }

      


        [TestMethod]
        public void TestConcurrentDeleteOrderItem()
        {
             Util.CreateEmptyOrder();

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                AssertNoChangesYet(ctx);

                foreach (Product p in ctx.ProductSet)
                {
                    order.OrderItems.Add(new OrderItem()
                    {
                        Quantity = 1,
                        Products = p,
                    });
                }


                ctx.Submit();
            }


            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version")
                    .First();

                AssertNoChangesYet(ctx);

                order.Shipped = true;

                using (AcmeWidgets ctx2 = new AcmeWidgets())
                {
                    Order order2 = ctx2.OrderSet
                        .Include("Version")
                        .Include("OrderItems")
                        .Include("OrderItems.Version")
                        .First();


                    ctx2.DeleteObject(order2.OrderItems.First());

                    ctx2.Submit();
                }


                try
                {
                    ctx.Submit();
                    Assert.Fail("Expected currency issue");

                }
                catch (OptimisticConcurrencyException error)
                {

                }
            }


        }


        [TestMethod]
        public void TestModifyOrder()
        {
            //
            // TODO: Add test logic	here
            //

             Util.CreateEmptyOrder();
            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                AssertNoChangesYet(ctx);

                order.Shipped = true;
                order.Version.Increment();

                ctx.SaveChanges();
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                Assert.IsTrue(order.Shipped, "Order was marked as shipped");
                Assert.AreEqual(1, order.Version.Value, "Expected version to be incremented");
            }


        }

        [TestMethod]
        public void TestConcurrentModifyOrder()
        {
            //
            // TODO: Add test logic	here
            //

             Util.CreateEmptyOrder();
            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();
                AssertNoChangesYet(ctx);

                order.Shipped = true;

                using (AcmeWidgets secondCtx = new AcmeWidgets())
                {
                    Order secondOrder = secondCtx.OrderSet.Include("Version").First();
                    AssertNoChangesYet(secondCtx);

                    secondOrder.Shipped = true;

                    secondCtx.Submit();
                }

                try
                {
                    ctx.Submit();
                    Assert.Fail("Expected to get a concurrency error");
                }
                catch (OptimisticConcurrencyException error)
                {

                }
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                Assert.IsTrue(order.Shipped, "Order was marked as shipped");
                Assert.AreEqual(1, order.Version.Value, "Expected version to be incremented");
            }


        }

        [TestMethod]
        public void TestAddOrderItemsConcurrency()
        {

             Util.CreateEmptyOrder();

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();
                Product firstProduct = ctx.ProductSet.First();

                order.OrderItems.Add(new OrderItem()
                {
                    Quantity = 1,
                    Products = firstProduct,
                });

                using (AcmeWidgets secondCtx = new AcmeWidgets())
                {
                    Order secondOrder = secondCtx.OrderSet.Include("Version").First();
                    Product lastProduct = secondCtx.ProductSet.ToList().Last();

                    secondOrder.OrderItems.Add(new OrderItem()
                    {
                        Quantity = 1,
                        Products = lastProduct,
                    });
               
                    secondCtx.Submit();
                }

                try
                {
                    ctx.Submit();
                    Assert.Fail("Expected to get a concurrency error");
                }
                catch (OptimisticConcurrencyException error)
                {

                }
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").Include("OrderItems").First();

                Assert.AreEqual(1 , order.OrderItems.Count, "Order was marked as shipped");
                Assert.AreEqual(1, order.Version.Value, "Expected version to be incremented");
            }

        }


        [TestMethod]
        public void TestModifyOrderItemsConcurrency()
        {

             Util.CreateEmptyOrder();
            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").First();

                foreach (Product p in ctx.ProductSet)
                {
                    order.OrderItems.Add(new OrderItem()
                    {
                        Quantity = 1,
                        Products = p,
                    });
                }


                ctx.Submit();
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet
                    .Include("Version")
                    .Include("OrderItems")
                    .Include("OrderItems.Version")
                    .First();

                order.OrderItems.First().Quantity++;

                using (AcmeWidgets secondCtx = new AcmeWidgets())
                {
                    Order secondOrder = secondCtx.OrderSet
                    .Include("Version")
                    .Include("OrderItems")
                    .Include("OrderItems.Version")
                    .First();

                    secondOrder.OrderItems.Last().Quantity++;

                    secondCtx.Submit();
                }

                try
                {
                    ctx.Submit();
                    Assert.Fail("Expected to get a concurrency error");
                }
                catch (OptimisticConcurrencyException error)
                {

                }
            }

            using (AcmeWidgets ctx = new AcmeWidgets())
            {
                Order order = ctx.OrderSet.Include("Version").Include("OrderItems").First();

                Assert.AreEqual(2, order.OrderItems.Count, "Order was marked as shipped");
                Assert.AreEqual(2, order.OrderItems.Last().Quantity, "Expected two items");
                Assert.AreEqual(2, order.Version.Value, "Expected version to be incremented twice");
            }

        }


        // Moving items between collections not really supported by EF
        // http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=388629
        //[TestMethod]
        //public void TestSeperateOrder()
        //{
        //    CreateEmptyOrder();
        //    using (AcmeWidgets ctx = new AcmeWidgets())
        //    {
        //        Order order = ctx.OrderSet.Include("Version").First();

        //        foreach (Product p in ctx.ProductSet)
        //        {
        //            order.OrderItems.Add(new OrderItem()
        //            {
        //                Quantity = 1,
        //                Products = p,
        //            });
        //        }


        //        ctx.Submit();
        //    }

        //    using (AcmeWidgets ctx = new AcmeWidgets())
        //    {
        //        Order order = ctx.OrderSet
        //            .Include("Version")
        //            .Include("OrderItems")
        //            .Include("OrderItems.Version")
        //            .First();

        //        Customer customer = new Customer()
        //        {
        //            Name = "Phil",
        //            City = "Cheltenham",
        //            Street = "High Street",
        //            Zip = "GL5",
        //        };


        //        Order secondOrder = new Order()
        //        {
        //            Customers = customer,
        //            Shipped = false,
        //            Version = new SharedVersion(),
        //        };

        //        ctx.AddToOrderSet(secondOrder);

        //       OrderItem lastItem = order.OrderItems.Last();

        //       order.OrderItems.Remove(lastItem);
      
        //        ctx.DeleteObject(lastItem);

        //        ctx.Submit();
        //    }


        //    using (AcmeWidgets ctx = new AcmeWidgets())
        //    {
        //        List<Order> orders = ctx.OrderSet
        //            .Include("Version")
        //            .Include("OrderItems")
        //            .ToList();

        //        foreach (Order order in orders)
        //        {
        //            Assert.AreEqual(1, order.OrderItems.Count());
        //        }
        //    }
        //}



        private static void AssertNoChangesYet(AcmeWidgets ctx)
        {
            var changes = ctx.ObjectStateManager.GetObjectStateEntries(EntityState.Added | EntityState.Deleted | EntityState.Modified).ToList();
            Assert.AreEqual(0, changes.Count(), "Didn't expect to see any changes");
        }
       
    }
}
