
use AcmeWidgetsEF
go
drop procedure DeleteAll

drop procedure CreateProduct
drop procedure UpdateProduct
drop procedure DeleteProduct
drop procedure GetAllProducts
drop procedure GetProduct

drop procedure AddProductToCateogry
drop procedure RemoveProductToCateogry

drop procedure CreateCustomer
drop procedure DeleteCustomer
drop procedure UpdateCustomer

drop procedure GetAllOpenOrders

drop procedure LockComposite
drop procedure UnLockComposite
drop procedure HasCompositeLock
drop procedure ReleaseAllCompositeLocks

drop table orderItems
drop table Orders
drop table ProductCategoriesMappings;
drop table ProductCategories
drop table Products
drop table Customers
drop table Versions

go

CREATE TABLE [dbo].[Versions] (
	[id] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
	[version] int not null,
	[expires] datetime,
	[owner] varchar(200),
	);

go

CREATE PROCEDURE LockComposite(
	@id int,
	@owner varchar(200),
	@version int
	) AS
BEGIN
	UPDATE Versions
	SET owner = @owner,
	expires = DATEADD( N , 30 , GETDATE() )
	where [version] = @version and id = @id 
	and ( (owner is null) or ( expires < GETDATE() ))
	
    IF @@ROWCOUNT != 1 RAISERROR('Failed to lock entity', 11,1);
	
END	
	
go

CREATE PROCEDURE UnLockComposite(
	@id int,
	@version int,
	@owner varchar(200)
	) AS
BEGIN
	UPDATE Versions
	SET owner = null,
	expires = null,
	[version] = @version + 1
	where id = @id and owner = @owner
	
END	
	
go

CREATE PROCEDURE HasCompositeLock( 
	@id int,
	@owner varchar(200) ) AS
	BEGIN
		select COUNT(*) from Versions where id = @id and owner = @owner
	END
	
go

CREATE PROCEDURE ReleaseAllCompositeLocks( 
	@owner varchar(200) ) AS
	BEGIN
		UPDATE Versions
	SET owner = null,
	expires = null
	where owner = @owner
	END
	
go

CREATE TABLE [dbo].[Products](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[version] rowversion not null,
	[Title] [varchar](200) NOT NULL,
	[Description] [varchar](500) NULL,
	[Price] [money] NOT NULL,
	[SalesTax] [money] NOT NULL,
 CONSTRAINT [PK_Products] PRIMARY KEY CLUSTERED 
 (
	[id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
 
go

CREATE PROCEDURE CreateProduct(
	@title varchar(200),
	@description varchar(200),
	@price money,
	@salesTax money )
As
BEGIN
	Insert into Products ( Title , Description , Price , SalesTax  )
	values ( @title , @description , @price , @salesTax )
	
	select * from Products where id=SCOPE_IDENTITY()
END

go

CREATE PROCEDURE GetAllProducts
As
	/* select id Id,title Name,description Description, price Price , salesTax Tax from Products */
	select * from Products

go

CREATE PROCEDURE GetProduct(
	@id int
	)
	As
	--select id Id , title Name,description Description, price Price , salesTax Tax 
	--from Products where id = @id
	select * from Products where id = @id

go

CREATE PROCEDURE DeleteProduct(
	@id int 
	)
	As
	
	delete Products where id=@id

go

CREATE PROCEDURE UpdateProduct(
	@id int,
	@version rowversion,
	@title varchar(200),
	@description varchar(200),
	@price money,
	@salesTax money )
As
	Update Products
	set Title =@title,
	Description =@description,
	Price =@price ,
	SalesTax = @salesTax
	where id= @id
	and version = @version
	
	if  @@rowcount = 0 
	BEGIN
		--RAISERROR('No such product or old version',11,1)
		return
	END
   	
	select version from Products
	where id =@id
	
go



/* ************* CUSTOMERS ************************* */
/****** Object:  Table [dbo].[Customers]    Script Date: 01/13/2009 15:51:17 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[Customers](
	[Id] [int] IDENTITY(1,1) NOT NULL primary key,
	[version] rowversion not null,
	[Name] [varchar](200) NOT NULL,
	[Street] [varchar](100) NOT NULL,
	[City] [varchar](100) NOT NULL,
	[Zip] [varchar](30) NULL
) ON [PRIMARY]
GO

CREATE PROCEDURE CreateCustomer(
	@name varchar(200),
	@street varchar(100),
	@city varchar(100),
	@zip varchar(30) )
As
BEGIN
	Insert into Customers ( Name , Street , City , Zip  )
	values ( @name , @street , @city , @zip )
	
	select * from Customers where id=SCOPE_IDENTITY()
END

go



CREATE PROCEDURE DeleteCustomer(
	@id int 
	)
	As
	
	delete Customers where id=@id

go

CREATE PROCEDURE UpdateCustomer(
	@id int,
	@version rowversion,
	@name varchar(200),
	@street varchar(100),
	@city varchar(100),
	@zip varchar(30) )
As
	Update Customers
	set Name =@name,
	City =@city,
	Street =@street ,
	Zip = @zip
	where id= @id and version = @version
	
	select version from Customers where Id=@id
go

/****** Object:  Table [dbo].[ProductCategories]    Script Date: 01/13/2009 15:51:17 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[ProductCategories]    (
	[Id] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
	[Title] [varchar](200) NOT NULL,
	[Description] [varchar](500) NOT NULL,
) ON [PRIMARY]
GO

/****** Object:  Table [dbo].[ProductCategoriesMappings]    Script Date: 01/13/2009 15:51:17 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[ProductCategoriesMappings](
	[ProductId] [int] NOT NULL,
	[ProductCategoryId][int] NOT NULL ,
	Constraint PK_PRODUCTCATEGORIESMAPPINGS PRIMARY KEY(ProductID,ProductCategoryId)
) ON [PRIMARY]
GO

CREATE PROCEDURE AddProductToCateogry(
	@productId int,
	@categoryId int )
AS
	INSERT INTO ProductCategoriesMappings ( productId , productCategoryId ) 
	VALUES ( @productId , @categoryId )
go

CREATE PROCEDURE RemoveProductToCateogry(
	@productId int,
	@categoryId int )
AS
	DELETE ProductCategoriesMappings
	where productId = @productId
	AND productCategoryId = @categoryId
go


/****** Object:  Table [dbo].[Orders]    Script Date: 01/13/2009 15:51:17 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TABLE [dbo].[Orders](
	[Id] [int] IDENTITY(1,1) NOT NULL ,
	[Customer] [int] NOT NULL,
	[Shipped] [bit] NOT NULL,
	[Version] [int] NOT NULL,
 CONSTRAINT [PK_Orders] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY],
CONSTRAINT FK_ORDERS_CUSTOMER FOREIGN KEY (Customer) References Customers(Id),
CONSTRAINT FK_ORDERS_VERSION FOREIGN KEY (Version) References Versions(Id)
) ON [PRIMARY]
GO

CREATE TABLE [dbo].[OrderItems](
	[OrderId] [int] NOT NULL ,
	[ProductId] [int] NOT NULL,
	[Quantity] [int] NOT NULL,
		[Version] [int] NOT NULL,
 CONSTRAINT PK_ORDERITEMS PRIMARY KEY ( OrderId,ProductId),
 CONSTRAINT FK_ORDERITEMS_ORDER FOREIGN KEY (OrderId) References Orders(Id),
 CONSTRAINT FK_ORDERITEMS_PRODUCT FOREIGN KEY (ProductId) References Products(id),
 CONSTRAINT FK_ORDERITEMSS_VERSION FOREIGN KEY (Version) References Versions(Id)
)

GO

CREATE PROCEDURE GetAllOpenOrders
As
	select * from Orders
	where Shipped = 0
go



CREATE PROCEDURE DeleteAll
As
	delete from OrderItems
	delete from ProductCategoriesMappings
	delete from Orders
	delete from Customers
	delete from Products
	delete from ProductCategories
	delete from Versions
go

/****** Object:  ForeignKey [FK_ProductCategoriesMappings_ProductCategories]    Script Date: 01/13/2009 15:51:17 ******/
ALTER TABLE [dbo].[ProductCategoriesMappings]  WITH CHECK ADD  CONSTRAINT [FK_ProductCategoriesMappings_ProductCategories] FOREIGN KEY([ProductCategoryId])
REFERENCES [dbo].[ProductCategories] ([Id])
GO
ALTER TABLE [dbo].[ProductCategoriesMappings] CHECK CONSTRAINT [FK_ProductCategoriesMappings_ProductCategories]
GO
/****** Object:  ForeignKey [FK_ProductCategoriesMappings_Products]    Script Date: 01/13/2009 15:51:17 ******/
ALTER TABLE [dbo].[ProductCategoriesMappings]  WITH CHECK ADD  CONSTRAINT [FK_ProductCategoriesMappings_Products] FOREIGN KEY([ProductId])
REFERENCES [dbo].[Products] ([id])
GO
ALTER TABLE [dbo].[ProductCategoriesMappings] CHECK CONSTRAINT [FK_ProductCategoriesMappings_Products]
GO